#version 150

layout (triangles) in;
layout (triangle_strip, max_vertices = 9) out;

uniform mat4 modelview;
uniform mat4 transform;
uniform float fuerza;

in VertData {
  vec4 color;
} vertData[];

out FragData {
  vec4 color;
} fragData;

void explode(int i, int j, vec3 d, vec4 cpos, vec4 ccol, 
             float f1, float f2, float f3) {
  gl_Position = transform * (gl_in[i].gl_Position + vec4(f1 * fuerza * d, 0));
  fragData.color = vertData[i].color;
  EmitVertex();
  
  // d = normalize(gl_in[1].gl_Position.xyz - cubeCenter);
  gl_Position = transform * (gl_in[j].gl_Position + vec4(f2 * fuerza * d, 0));
  fragData.color = vertData[j].color; 
  EmitVertex();

  // d = normalize(center.xyz - cubeCenter);
  gl_Position = transform * (cpos + vec4(f3 * fuerza * d, 0));
  fragData.color = ccol;
  EmitVertex();

  EndPrimitive();
}

void main() {
  vec3 cubeCenter = vec3(0, 0, -519);

  vec4 sumPos = vec4(0);
  vec4 sumCol = vec4(0);
  for (int i = 0; i < gl_in.length(); i++) {
    vec4 pos = gl_in[i].gl_Position;
    vec4 col = vertData[i].color;
    sumPos += pos;
    sumCol += col;
  }
  vec4 center = sumPos / gl_in.length();
  vec4 centerColor = sumCol / gl_in.length();

  // Desplazamiento
  vec3 d = normalize(center.xyz - cubeCenter);

  // Triangulo 1
  explode(0, 1, d, center, centerColor, 0.3, 1.2, 0.8);  

  // Triangulo 2
  explode(1, 2, d, center, centerColor, 0.9, 0.45, 0.3);

  // Triangulo 3
  explode(2, 0, d, center, centerColor, 1.6, 0.34, 0.65);
}